package org.openmuc.core.common;

/*
 * Copyright Fraunhofer ISE, 2011
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import gnu.io.SerialPort;

public class SerialPortSettings {
	private int baudrate;
	private int dataBits;
	private int stopBits;
	private int parity;
	
	/**
	 * Parse a string containing serial port settings
	 * 
	 * @param settings is a string in form BBBB-SPE e.g. 2400-8N1 9600-7E2 19200-8O1
	 */
	public SerialPortSettings(String settings) throws IllegalArgumentException {
		int baudDelimiter = settings.indexOf('-');
		
		if (baudDelimiter == -1) throw new IllegalArgumentException(settings);
		
		String baudrate = settings.substring(0, baudDelimiter);
		
		try {
			this.baudrate = Integer.parseInt(baudrate);
		} catch (Exception e) {
			throw new IllegalArgumentException("Baudrate should be a valid number!");
		}
		
		char dataBits = settings.charAt(baudDelimiter + 1);
		char parity = settings.charAt(baudDelimiter + 2);		
		char stopBits = settings.charAt(baudDelimiter + 3);
		System.out.println(dataBits);
		System.out.println(parity);
		System.out.println(stopBits);
		parseParityCharacter(parity);
		parseDataBitsCharacter(dataBits);		
		parseStopBitsCharacter(stopBits);
	}

	private void parseStopBitsCharacter(char stopBits) {
		switch (stopBits) {
		case '1':
			this.stopBits = SerialPort.STOPBITS_1;
			break;
		case '2':
			this.stopBits = SerialPort.STOPBITS_2;
			break;
		default:
			throw new IllegalArgumentException("Unknown stop bits character");
		}
	}

	private void parseDataBitsCharacter(char dataBits) {
		switch (dataBits) {
		case '5':
			this.dataBits = SerialPort.DATABITS_5;
			break;
		case '6':
			this.dataBits = SerialPort.DATABITS_6;
			break;
		case '7':
			this.dataBits = SerialPort.DATABITS_7;
			break;
		case '8':
			this.dataBits = SerialPort.DATABITS_8;
			break;
		default:
			throw new IllegalArgumentException("Unknown data bits character");	
		}
	}

	private void parseParityCharacter(char parity) {
		switch (parity) {
		case 'e':
		case 'E':
			this.parity = SerialPort.PARITY_EVEN;
			break;
		case 'o':
		case 'O':
			this.parity = SerialPort.PARITY_ODD;
			break;
		case 'n':
		case 'N':
			this.parity = SerialPort.PARITY_NONE;
			break;
		case 'm':
		case 'M':
			this.parity = SerialPort.PARITY_MARK;
			break;
		case 's':
		case 'S':
			this.parity = SerialPort.PARITY_SPACE;
			break;
		default:
			throw new IllegalArgumentException("Unknown parity character");	
		}
	}
	
	public int getBaudrate() {
		return baudrate;
	}
	public int getDataBits() {
		return dataBits;
	}
	public int getStopBits() {
		return stopBits;
	}
	public int getParity() {
		return parity;
	}
	
	public void setBaudrate(int baudrate) {
		this.baudrate = baudrate;
	}
	public void setDataBits(int dataBits) {
		this.dataBits = dataBits;
	}
	public void setStopBits(int stopBits) {
		this.stopBits = stopBits;
	}
	public void setParity(int parity) {
		this.parity = parity;
	}
}
