package org.openmuc.core.datamanager;

import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.State;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * @author mzillgit
 *
 * Bundles can be informed by the data manager service if an event occurs.
 * This is implemented using the "whiteboard" pattern.  Each bundle
 * has to register a DataManagerListener service to get informed.
 *
 */

public interface DataManagerListener {
	
	/**
	 * Invoked if a configuration update occurs
	 */
	public void updated();
	
	/**
	 * Invoked if the state of a channel has changed
	 * 
	 * @param id id of the channel that changed its state
	 * @param newState new state of the channel
	 */
	public void stateChanged(ChannelIdentifier id, State newState);
		
	/**
	 * Invoked if the framework wants to store data
	 * 
	 * @param string
	 * @param timestamp
	 * @param value
	 */
	@Deprecated
	public void storingDataEvent(String string, long timestamp, double value);
}
