package org.openmuc.core.datamanager.config;

import java.util.List;

public interface ConfigurationService {

	/**
	 * Register a listener to get informed when the configuration changes.
	 * 
	 * @param listener
	 */
	public abstract void registerListener(ConfigurationListener listener);

	/**
	 * Saves the configuration to the persistent storage
	 */
	public abstract void persist();

	/**
	 * Creates a new DeviceConfiguration object for a device and add it to the
	 * configuration.
	 * 
	 * @param name
	 *            unique name of the new meter
	 * @return MeterConfiguration object for the new meter
	 */
	public abstract DeviceConfiguration createDevice(String name);

	/**
	 * Reloads configuration from persistent storage
	 */
	public abstract void reloadConfiguration();

	/**
	 * Returns an immutable list of configured meters. To add or delete a meter
	 * use the createDevice() and deleteDevice() functions.
	 */
	public abstract List<DeviceConfiguration> getDevices();

	/**
	 * Returns the DeviceConfiguration object for the device with the specified
	 * name.
	 * 
	 * @param name
	 *            unique name of the device
	 * @return DeviceConfiguration object for the device
	 */
	public abstract DeviceConfiguration getDeviceByName(String name);

	/**
	 * Delete a device from the configuration.
	 * 
	 * @param name
	 *            unique name of the device
	 */
	public abstract void deleteDevice(String name);

	public abstract PlantConfiguration getPlantConfiguration();
}