package org.openmuc.core.driver;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * The class ChannelInformation provides Information about a certain channel<br>
 * It's used in Drivers to return Information about the Measurement Channels. It
 * may contain:<br>
 * <b>label, DLMS Unit Code, current Value</b>
 */
public class ChannelInformation {
	private String name;
	private String description;
	private int dlmsUnitCode;

	/**
	 * Creates an Instance of ChannelInformation
	 */
	public ChannelInformation(String name) {
		this.name = name;
	}

	/**
	 * Creates an Instance of ChannelInformation
	 * 
	 */
	public ChannelInformation(String name, int DLMSUnitCode) {
		this(name);
		this.dlmsUnitCode = DLMSUnitCode;
	}

	/**
	 * Creates an Instance of ChannelInformation
	 * 
	 */
	public ChannelInformation(String name, int DLMSUnitCode, String description) {
		this(name, DLMSUnitCode);
		this.description = description;
	}

	/**
	 * Gets the label of this channel.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Gets the DLMS Unit Code
	 */
	public int getDLMSUnitCode() {
		return dlmsUnitCode;
	}

	/**
	 * Gets the current Measurement Value
	 */
	public String getDescription() {
		return description;
	}
}
