package org.openmuc.core.driver;

import java.util.HashMap;
import java.util.Map;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

public class DeviceURI {
	private String driverID = null;
	private String comPort = null;
	private String address = null;
	private String settings = null;
	
	private Map<String,String> parameters = new HashMap<String,String>();
	
	public DeviceURI(String url) {
		int pos;
		String rest;
		
		if ((pos = url.indexOf(':')) >= 0) {
			driverID = url.substring(0, pos);
			rest = url.substring(pos + 1);
			if ((pos = rest.indexOf(':')) >= 0) {
				comPort = rest.substring(0, pos);
				rest = rest.substring(pos + 1);
				if ((pos = rest.indexOf('?')) >= 0) {
					address = rest.substring(0, pos);
					settings = rest.substring(pos + 1);
					parseSetting(settings);
				}
				else if (rest.length() > 0) {
					address = rest;
				}
			}
			else comPort = rest;
		}
	}
	
	private void parseSetting(String settings2) {
		String[] parameter = settings2.split(";");
		
		String key = null;
		String value = null;
		
		for (int i = 0; i < parameter.length; i++) {
			String[] keyValue = parameter[i].split("=");
			key = keyValue[0];
			
			if (keyValue.length > 1) {
				value = keyValue[1];
			} else value = null;
			
			parameters.put(key, value);
		}
	}
	
	public String getParameter(String key) {
		return parameters.get(key);
	}
	
	public String toString() {
		StringBuffer buf = new StringBuffer(50);
		
		if (driverID != null) buf.append(driverID);
		buf.append(':');
		if (comPort != null) buf.append(comPort);
		
		if (address != null) {
			buf.append(':');
			buf.append(address);
		}
		if (settings != null) {
			buf.append('?');
			buf.append(settings);
		}
		
		return buf.toString();
	} /* toString() */
	
	public String getDriverID() {
		return driverID;
	}
	public String getComPort() {
		return comPort;
	}
	public String getAddress() {
		return address;
	}
	public String getSettings() {
		return settings;
	}
}
