package org.openmuc.core.measurement;

/*
 * Copyright Fraunhofer ISE, 2011
 * Author(s): Michael Zillgith
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * 
 * This enum represents a measurement unit. It should be compatible with the DLMS
 * unit codes according to IEC 62056-62. It is aimed to replace the former DLMSUnit class
 * 
 * @author mzillgit
 *
 */
public enum Unit {
	
	/* Unit codes are compatible to DLMS unit codes according to IEC 62056-62. */
	UNKNOWN(0),
	YEAR(1), MONTH(2), WEEK(3), DAY(4), HOUR(5), MIN(6), SECOND(7),
	DEGREE(8), DEGREE_CELSIUS(9), CURRENCY(10), METRE(11), METRE_PER_SECOND(12),
	CUBIC_METRE(13), CUBIC_METRE_CORRECTED(14), CUBIC_METRE_PER_HOUR(15),
	CUBIC_METRE_PER_HOUR_CORRECTED(16), CUBIC_METRE_PER_DAY(17),
	CUBIC_METRE_PER_DAY_CORRECTED(18), LITRE(19), KILOGRAM(20), NEWTON(21),
	NEWTONMETER(22), PASCAL(23), BAR(24), JOULE(25), JOULE_PER_HOUR(26), WATT(27),
	VOLT_AMPERE(28), VAR(29), WATT_HOUR(30), VOLT_AMPERE_HOUR(31), VAR_HOUR(32),
	AMPERE(33), COULOMB(34), VOLT(35), VOLT_PER_METRE(36), FARAD(37), OHM(38),
	OHM_METRE(39), WEBER(40), TESLA(41), AMPERE_PER_METRE(42), HENRY(43), HERTZ(44),
	ACTIVE_ENERGY_METER_CONSTANT_OR_PULSE_VALUE(45),
	REACTIVE_ENERGY_METER_CONSTANT_OR_PULSE_VALUE(46),
	APPARENT_ENERGY_METER_CONSTANT_OR_PULSE_VALUE(47),
	VOLT_SQUARED_HOURS(48), AMPERE_SQUARED_HOURS(49), KILOGRAM_PER_SECOND(50),
	KELVIN(52), VOLT_SQUARED_HOUR_METER_CONSTANT_OR_PULSE_VALUE(53),
	AMPERE_SQUARED_HOUR_METER_CONSTANT_OR_PULSE_VALUE(54),
	METER_CONSTANT_OR_PULSE_VALUE(55), PERCENTAGE(56), AMPERE_HOUR(57),
	ENERGY_PER_VOLUME(60), CALORIFIC_VALUE(61), MOLE_PERCENT(62), MASS_DENSITY(63),
	PASCAL_SECOND(64), RESERVED(253), OTHER_UNIT(254), COUNT(255),
	/* not mentioned in 62056, added for MBus: */
	CUBIC_METRE_PER_SECOND(150), CUBIC_METRE_PER_MINUTE(151), KILOGRAM_PER_HOUR(152);
		
	private int unitCode;
	
	private Unit (int unitCode) {
		this.unitCode = unitCode;
	}
	
	/**
	 * Returns the code of the unit according to DLMS.
	 * 
	 * @return unit code as int
	 */
	public int getUnitCode() {
		return unitCode;
	}
	
	
	public static Unit fromUnitCode(int code) {
		switch (code) {
		case 1:
			return Unit.YEAR;
		case 2:
			return Unit.MONTH;
		case 3:
			return Unit.WEEK;
		case 4:
			return Unit.DAY;
		case 5:
			return Unit.HOUR;
		case 6:
			return Unit.MIN;
		case 7:
			return Unit.SECOND;
		case 8:
			return Unit.DEGREE;
		case 9:
			return Unit.DEGREE_CELSIUS;
		case 10:
			return Unit.CURRENCY;
		case 11:
			return Unit.METRE;
		case 12:
			return Unit.METRE_PER_SECOND;
		case 13:
			return Unit.CUBIC_METRE;
		case 14:
			return Unit.CUBIC_METRE_CORRECTED;
		case 15:
			return Unit.CUBIC_METRE_PER_HOUR;
		case 16:
			return Unit.CUBIC_METRE_PER_HOUR_CORRECTED;
		case 17:
			return Unit.CUBIC_METRE_PER_DAY;
		case 18:
			return Unit.CUBIC_METRE_PER_DAY_CORRECTED;
		case 19:
			return Unit.LITRE;
		case 20:
			return Unit.KILOGRAM;
		case 21:
			return Unit.NEWTON;
		case 22:
			return Unit.NEWTONMETER;
		case 23:
			return Unit.PASCAL;
		case 24:
			return Unit.BAR;
		case 25:
			return Unit.JOULE;
		case 26:
			return Unit.JOULE_PER_HOUR;
		case 27:
			return Unit.WATT;
		case 28:
			return Unit.VOLT_AMPERE;
		case 29:
			return Unit.VAR;
		case 30:
			return Unit.WATT_HOUR;
		case 31:
			return Unit.VOLT_AMPERE_HOUR;
		case 32:
			return Unit.VAR_HOUR;
		case 33:
			return Unit.AMPERE;
		case 34:
			return Unit.COULOMB;
		case 35:
			return Unit.VOLT;
		case 36:
			return Unit.VOLT_PER_METRE;
		case 37:
			return Unit.FARAD;
		case 38:
			return Unit.OHM;
		case 39:
			return Unit.OHM_METRE;
		case 40:
			return Unit.WEBER;
		case 41:
			return Unit.TESLA;
		case 42:
			return Unit.AMPERE_PER_METRE;
		case 43:
			return Unit.HENRY;
		case 44:
			return Unit.HERTZ;
		case 45:
			return Unit.ACTIVE_ENERGY_METER_CONSTANT_OR_PULSE_VALUE;
		case 46:
			return Unit.REACTIVE_ENERGY_METER_CONSTANT_OR_PULSE_VALUE;
		case 47:
			return Unit.APPARENT_ENERGY_METER_CONSTANT_OR_PULSE_VALUE;
		case 48:
			return Unit.VOLT_SQUARED_HOURS;
		case 49:
			return Unit.AMPERE_SQUARED_HOURS;
		case 50:
			return Unit.KILOGRAM_PER_SECOND;
		case 52:
			return Unit.KELVIN;
		case 53:
			return Unit.VOLT_SQUARED_HOUR_METER_CONSTANT_OR_PULSE_VALUE;
		case 54:
			return Unit.AMPERE_SQUARED_HOUR_METER_CONSTANT_OR_PULSE_VALUE;
		case 55:
			return Unit.METER_CONSTANT_OR_PULSE_VALUE;
		case 56:
			return Unit.PERCENTAGE;
		case 57:
			return Unit.AMPERE_HOUR;
		case 60:
			return Unit.ENERGY_PER_VOLUME;
		case 61:
			return Unit.CALORIFIC_VALUE;
		case 62:
			return Unit.MOLE_PERCENT;
		case 63:
			return Unit.MASS_DENSITY;
		case 64:
			return Unit.PASCAL_SECOND;
		case 253:
			return Unit.RESERVED;
		case 254:
			return Unit.OTHER_UNIT;
		case 255:
			return Unit.COUNT;
		case 150:
			return CUBIC_METRE_PER_SECOND;
		case 151:
			return CUBIC_METRE_PER_MINUTE;
		case 152:
			return KILOGRAM_PER_HOUR;
			
		default:
			return Unit.UNKNOWN;
		}		
	}
	
	public String toString() {
		return getString(this);
	}
	
	/*
	 *  TODO make compatible to UCUM (http://unitsofmeasure.org/)
	 */
		
	public static String getString(Unit unit) {
		switch (unit) {
		case YEAR:
			return "a";
		case MONTH:
			return "mo";
		case WEEK:
			return "wk";
		case DAY:
			return "d";
		case HOUR:
			return "h";
		case MIN:
			return "min";
		case SECOND:
			return "s";
		case DEGREE:
			return "°";
		case DEGREE_CELSIUS:
			return "°C";
		case CURRENCY:
			return "currency";
		case METRE:
			return "m";
		case METRE_PER_SECOND:
			return "m/s";
		case CUBIC_METRE:
			return "m^3";
		case CUBIC_METRE_CORRECTED:
			return "m^3";
		case CUBIC_METRE_PER_HOUR:
			return "m^3/h";
		case CUBIC_METRE_PER_HOUR_CORRECTED:
			return "m^3/h";
		case CUBIC_METRE_PER_DAY:
			return "m^3/d";
		case CUBIC_METRE_PER_DAY_CORRECTED:
			return "m^3/d";
		case LITRE:
			return "l";
		case KILOGRAM:
			return "kg";
		case NEWTON:
			return "N";
		case NEWTONMETER:
			return "Nm";
		case PASCAL:
			return "Pa";
		case BAR:
			return "bar";
		case JOULE:
			return "J";
		case JOULE_PER_HOUR:
			return "J/h";
		case WATT:
			return "W";
		case VOLT_AMPERE:
			return "VA";
		case VAR:
			return "var";
		case WATT_HOUR:
			return "Wh";
		case VOLT_AMPERE_HOUR:
			return "VAh";
		case VAR_HOUR:
			return "varh";
		case AMPERE:
			return "A";
		case COULOMB:
			return "C";
		case VOLT:
			return "V";
		case VOLT_PER_METRE:
			return "V/m";
		case FARAD:
			return "F";
		case OHM:
			return "Ohm";
		case OHM_METRE:
			return "Ohm m";
		case WEBER:
			return "Wb";
		case TESLA:
			return "T";
		case AMPERE_PER_METRE:
			return "A/m";
		case HENRY:
			return "H";
		case HERTZ:
			return "Hz";
		case ACTIVE_ENERGY_METER_CONSTANT_OR_PULSE_VALUE:
			return "1/(Wh)";
		case REACTIVE_ENERGY_METER_CONSTANT_OR_PULSE_VALUE:
			return "1/(varh)";
		case APPARENT_ENERGY_METER_CONSTANT_OR_PULSE_VALUE:
			return "1/(VAh)";
		case VOLT_SQUARED_HOURS:
			return "V^2h";
		case AMPERE_SQUARED_HOURS:
			return "A^2h";
		case KILOGRAM_PER_SECOND:
			return "kg/s";
		case KELVIN:
			return "K";
		case VOLT_SQUARED_HOUR_METER_CONSTANT_OR_PULSE_VALUE:
			return "1/(V^2h)";
		case AMPERE_SQUARED_HOUR_METER_CONSTANT_OR_PULSE_VALUE:
			return "1/(A^2h)";
		case METER_CONSTANT_OR_PULSE_VALUE:
			return "1/m^3";
		case PERCENTAGE:
			return "%";
		case AMPERE_HOUR:
			return "Ah";
		case ENERGY_PER_VOLUME:
			return "Wh/m^3";
		case CALORIFIC_VALUE:
			return "J/m^3";
		case MOLE_PERCENT:
			return "Mol %";
		case MASS_DENSITY:
			return "g/m^3";
		case PASCAL_SECOND:
			return "Pa s";
		case RESERVED:
			return "reserved";
		case OTHER_UNIT:
			return "other";
		case COUNT:
			return "count";
		case CUBIC_METRE_PER_SECOND:
			return "m^3/s";
		case CUBIC_METRE_PER_MINUTE:
			return "m^3/m";
		case KILOGRAM_PER_HOUR:
			return "kg/h";
		default:
			return "?";
		}
	}
}
