package org.openmuc.mux.dbprovider.slotsdb;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

public interface SlotsDatabase {

	/*
	 * File Extension for Slots Database Files.
	 * Only these Files will be loaded. 
	 */
	public static String FILE_EXTENSION = ".slots";
	
	/*
	 * Root Folder for Slots Database Files
	 */
	public static String DB_ROOT_FOLDER = System.getProperty(SlotsDatabase.class.getPackage().getName().toLowerCase() + ".dbfolder");
	
	/*
	 * If no other root folder is defined, data will be stored to this folder
	 */
	public static String DEFAULT_DB_ROOT_FOLDER = "slotsdbdata/";
		
	/*
	 * Root Folder for JUnit Testcases
	 */
	public static String DB_TEST_ROOT_FOLDER = "testdata/";
	
	/*
	 * limit open files in Hashmap
	 * 
	 * Default Linux Configuration:
	 * (should be below)
	 * 
	 * host:/#> ulimit -aH
	 * [...]
	 * open files                      (-n) 1024
	 * [...] 
	 */
	public static String MAX_OPEN_FOLDERS = System.getProperty(SlotsDatabase.class.getPackage().getName().toLowerCase() + ".max_open_folders");
	public static int MAX_OPEN_FOLDERS_DEFAULT = 512;
	
	/*
	 * configures the data flush period. The less you flush, the faster SLOTSDB will be.
	 * unset this System Property (or set to 0) to flush data directly to disk.
	 */
	public static String FLUSH_PERIOD=System.getProperty(SlotsDatabase.class.getPackage().getName().toLowerCase() + ".flushperiod");
	
	/*
	 * configures how long data will at least be stored in the SLOTSDB.
	 */
	public static String DATA_LIFETIME_IN_DAYS=System.getProperty(SlotsDatabase.class.getPackage().getName().toLowerCase() + ".limit_days");
	
	/*
	 * configures the maximum Database Size (in MB).
	 */
	public static String MAX_DATABASE_SIZE=System.getProperty(SlotsDatabase.class.getPackage().getName().toLowerCase() + ".limit_size");
	
	/*
	 * Minimum Size for SLOTSDB (in MB).
	 */
	public static int MINIMUM_DATABASE_SIZE = 2;
	
	/*
	 * Initial delay for scheduled tasks (sizewatcher, data expiration, etc.)
	 */
	public static int INITIAL_DELAY = 10000;
	
	/*
	 * Intervall for scanning expired, old data.
	 * Set this to 86400000 to scan every 24 hours.
	 */
	public static int DATA_EXPIRATION_CHECK_INTERVAL = 5000;
	
}
