package org.openmuc.canopen.driver;

/*
 * Copyright Fraunhofer ISE, 2011
 * Author(s): Frederic Robra
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.io.IOException;
import java.util.Hashtable;
import java.util.List;

import org.apache.log4j.Logger;
import org.openmuc.canopen.exception.NoDeviceException;
import org.openmuc.core.driver.BusURI;
import org.openmuc.core.driver.ChannelInformation;
import org.openmuc.core.driver.DataReceivedListener;
import org.openmuc.core.driver.DeviceDriverInterface;
import org.openmuc.core.driver.DeviceURI;
import org.openmuc.core.driver.MethodNotImplementedException;
import org.openmuc.core.driver.NoSuchInterfaceException;
import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.Measurement;

public class CANopenDriver implements DeviceDriverInterface {
	private static final String DRIVER_ID = "canopen";
	private static Logger logger = Logger.getLogger(CANopenDriver.class);
	private Hashtable<String, CANopenConnection> interfaces;
	
	public CANopenDriver(){
		interfaces = new Hashtable<String, CANopenConnection>();
	}
	
	public void configureInterface(String iface, String configStr)
			throws NoSuchInterfaceException, IOException {
		if(interfaces.get(iface) == null){
			logger.debug("Configure interface: " + iface);
			try {
				interfaces.put(iface, new CANopenConnection(iface, configStr));
			} catch (NoDeviceException e) {
				logger.error(e.getMessage());
				throw new NoSuchInterfaceException();
			}
		}
		
	}

	public List<ChannelInformation> getDeviceChannels(DeviceURI url)
			throws MethodNotImplementedException, IOException {
		throw new MethodNotImplementedException();
	}

	public List<DeviceURI> getDeviceList(BusURI url) 
			throws MethodNotImplementedException, IOException{
		throw new MethodNotImplementedException();
	}

	public String getDriverID() {
		return DRIVER_ID;
	}

	public List<String> getSupportedOperations() {
		return null;
	}

	public void readDeviceAsync(DeviceURI url, List<ChannelIdentifier> channels,
			DataReceivedListener callback, Object handle)
			throws MethodNotImplementedException {
		CANopenConnection connection = interfaces.get(url.getComPort());
		
		if(connection == null){
			try{
				configureInterface(url.getComPort(), url.getSettings());
				connection = interfaces.get(url.getComPort());
			}
			catch(Exception e){
				logger.error(e.getMessage());
				Thread thread = new Thread(new ErrorThread(callback, handle, DeviceDriverInterface.IO_ERROR));
				thread.start();
				return;
			}
		}
		if(!connection.isAlive()){
			connection.start();
		}
		connection.addReadoutTask(callback, handle, channels, url);
	}
	public List<Measurement> readDeviceSync(DeviceURI url,
			List<ChannelIdentifier> channels)
			throws MethodNotImplementedException, IOException {
		throw new MethodNotImplementedException();
	}

	public void writeChannel(DeviceURI url, ChannelIdentifier channel, double value)
			throws MethodNotImplementedException, IOException {
		throw new MethodNotImplementedException();
	}

	public void shutdown() {
		for(CANopenConnection connection : interfaces.values()){
			connection.close();
		}
		
	}
}
