package org.openmuc.canopen.lib;

public class SDOClient {
	
	private final int STATE_IDLE = 0;
	private final int STATE_UPLOAD_INIT_REQ_SENT = 1;
	private final int STATE_UPLOAD_REQ_NEXT = 2;
	private final int STATE_UPLOAD_RES_NEXT = 3;
	private final int STATE_UPLOAD_FINISHED_SUCCESSFUL = 4;
	
	private int baseCobID;
	private int state = STATE_IDLE;
	private int currentActionNode;
	private CANopenLib connection;
	
	private byte[] uploadData = null;
	private int uploadDataPointer;
	private int uploadRemainingBytes;
	private boolean toggle = false;
	
	protected SDOClient(int baseCobID, CANopenLib connection) {
		this.baseCobID = baseCobID;
		this.connection = connection;
	}
	
	public byte[] uploadData(int node, int index, int subIndex) {
		if (state != STATE_IDLE) {
			//TODO send abort?
			state = STATE_IDLE;
		}
		
		currentActionNode = node;
		
		connection.sendSDOUploadInitiate((byte) node, index, (short) subIndex);
		
		state = STATE_UPLOAD_INIT_REQ_SENT;
		
		while (state == STATE_UPLOAD_INIT_REQ_SENT) {
			try { Thread.sleep(1);} catch (InterruptedException e) {}
		}
		
		toggle = false;
		
		while (state != STATE_UPLOAD_FINISHED_SUCCESSFUL) {
		

			if (state == STATE_UPLOAD_REQ_NEXT) {
				connection.sendSDOUploadSegment((byte) node, toggle);
				
				state = STATE_UPLOAD_RES_NEXT;
			}
			else {
				System.err.println("Illegal state: " + state);
			}
		
			while (state == STATE_UPLOAD_RES_NEXT) {
				try { Thread.sleep(1);} catch (InterruptedException e) {}
			}
			
		}
		
		if (state == STATE_UPLOAD_FINISHED_SUCCESSFUL) {
			state = STATE_IDLE;
			return uploadData;
		}
		
		return null;
	}
	
	private void uploadResponseReceived(CLibrary.can_frame.ByReference frame) {
		long length;
		
		//TODO check for abort!
		
		switch (state) {
		case STATE_UPLOAD_INIT_REQ_SENT:
			if ((frame.data[0] & 0xf0) == 0x40) { /* upload initiate confirmation */
				
				switch (frame.data[0] & 0x03) {
				
				case 0x01: /* segmented message: this part only contains data length */
					length = frame.data[4] + frame.data[5] * 256 + 
					  		 frame.data[6] * 65536 + frame.data[7] * 16777216;
		
					uploadData = new byte[(int) length];
					uploadRemainingBytes = (int) length;
					uploadDataPointer = 0;
					state = STATE_UPLOAD_REQ_NEXT;
					break;
				
				case 0x03: /* unsegmented message - complete data included */
					length = 4 - ((frame.data[0] & 0x0c) >> 2);
					uploadData = new byte[(int) length];
					uploadDataPointer = 0;
					for (int i = 0; i < length; i++) {
						uploadData[i] = frame.data[4 + i];
						uploadDataPointer++;
					}
					state = STATE_UPLOAD_FINISHED_SUCCESSFUL;
					break;
				}
				
			}
			else
				System.err.println("Illegal response from server!");
			break;
		case STATE_UPLOAD_RES_NEXT:
			if ((frame.data[0] & 0xe0) == 0) {				
				//TODO check toggle bit
				length = 7 - ((frame.data[0] & 0x0e) >> 1);
				System.out.println("UPLOAD RES length: " + length);
				for (int i = 0; i < length; i++) {
					uploadData[uploadDataPointer] = frame.data[i + 1];
					uploadDataPointer++;
					uploadRemainingBytes--;
				}
				toggle = !toggle;
				
				if ((frame.data[0] & 0x01) == 1) {
					state = STATE_UPLOAD_FINISHED_SUCCESSFUL;
				}
				else {
					state = STATE_UPLOAD_REQ_NEXT;
				}
			}
			else
				System.err.println("A11: Illegal resonse from server!");
				
				
		}
	}
	
	protected boolean handleFrame(CLibrary.can_frame.ByReference frame) {
		if (frame.can_id == (0x580 + baseCobID)) {
			uploadResponseReceived(frame);
			return true;
		}
		else
			return false;
	}
}
