package org.openmuc.ehz.iec62056_21;


import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.openmuc.core.measurement.Unit;
import org.openmuc.core.obis.OBISCode;
import org.openmuc.ehz.driver.Device;
import org.openmuc.ehz.driver.DeviceInfo;
import org.openmuc.ehz.exception.NotFoundException;


public class IECDevice extends Device {

	private final static int MS_TIMEOUT = 3000;
	private String ID;
	private String name;

	private IECReceiver receiver;

	public IECDevice(String iface, String config) throws NotFoundException, IOException {
		super(iface, config);
		receiver = new IECReceiver(iface);
	}
	
	@Override
	public void close() {
		receiver.close();
	}

	@Override
	public Hashtable<String, Double> getValues() throws ParseException, IOException {
		Hashtable<String, Double> values =  new Hashtable<String, Double>();
		byte[] frame = receiver.receiveMessage(MS_TIMEOUT);
		ModeDMessage message = new ModeDMessage(frame);
		message.parse();
		List<String> dataSets = message.getDataSets();
		
		for (String dataSet : dataSets) {
			DataSet ds = new DataSet(dataSet);
			values.put(ds.getAddress(), ds.getVal());
		}
		
		return values;
	}

	@Override
	public List<DeviceInfo> getDeviceInfos() throws ParseException, IOException {
		List<DeviceInfo> infos = new ArrayList<DeviceInfo>();
		byte[] frame = receiver.receiveMessage(MS_TIMEOUT);
		ModeDMessage message = new ModeDMessage(frame);
		message.parse();
		
		ID = message.getIdentifier();
		name = message.getVendorID();

		List<String> dataSets = message.getDataSets();
		
		for (String dataSet : dataSets) {
			DataSet ds = new DataSet(dataSet);
			DeviceInfo info = new DeviceInfo();
			info.setAddress(ds.getAddress());
			info.setValue(ds.getVal());
			OBISCode code = new OBISCode(ds.getAddress());
			
			if (ds.getUnit() != null) {
				info.setDescription(ds.getUnit() + " " + code.getDescription(1));
				info.setUnit(Unit.UNKNOWN);
				String registerUnit = ds.getUnit().split(" ")[0];
				for (Unit x : Unit.values()) {
					if (x.toString().equals(registerUnit)) {
						info.setUnit(x);
						break;
					}
				}
			}
			else {
				info.setDescription(ds.getValue() + " " + code.getDescription(1));
				info.setUnit(Unit.UNKNOWN);
			}
			infos.add(info);
		}
		
		return infos;
	}

	@Override
	public String getID() {
		return ID;
	}

	@Override
	public String getName() {
		return name;
	}
}
