package org.openmuc.mbus.driver;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.openmuc.core.driver.BusURI;
import org.openmuc.core.driver.ChannelInformation;
import org.openmuc.core.driver.DeviceDriverInterface;
import org.openmuc.core.driver.DeviceURI;
import org.openmuc.core.driver.DataReceivedListener;
import org.openmuc.core.driver.MethodNotImplementedException;
import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.Measurement;
import org.openmuc.core.obis.OBISData;

public class MBusDriverImpl implements DeviceDriverInterface {
	private static Logger logger = Logger.getLogger(MBusDriverImpl.class);
	
	private static final String DRIVER_ID = "mbus";

	private Hashtable<String, MBusConnection> interfaces;

	public MBusDriverImpl() {
		interfaces = new Hashtable<String, MBusConnection>();
	}

	public int addMeter(String meterURL) {
		return 0;
	}

	public String getDriverID() {
		return DRIVER_ID;
	}

	public Vector<OBISData> getMeterValues(int valueSelectionIndicator) {
		return null;
	}

	public void getMeterValuesRequest(DataReceivedListener listener, 
			int valueSelectionIndicator)
	{
	}

	public Vector<String> getSupportedOperations() {
		return null;
	}

	/**
	 * 
	 * @param url
	 *            URL of the meter
	 * 
	 * @param registers
	 *            List of registers to readout. If registers == null all
	 *            registers will be read.
	 * 
	 * @param callback
	 *            Callback object if task finished or error/timeout occured.
	 * @throws IOException
	 */
	public void readDeviceAsync(DeviceURI url, List<ChannelIdentifier> registers, 
			DataReceivedListener callback,	Object handle) 
	{

		MBusConnection connection = interfaces.get(url.getComPort());

		if (connection == null) {
			try {
				configureInterface(url.getComPort(), "");
				connection = interfaces.get(url.getComPort());
						
			} catch (IOException e) {
				Thread thread = new Thread(new ErrorThread(callback, handle, DeviceDriverInterface.IO_ERROR));
				thread.start();
				logStackTrace(e);
				return;
			}
		}
		
		connection.addReadoutTask(callback, handle, registers, url);
	}

	/**
	 * @param iface
	 *            (eg. "usb0", "com1" etc)
	 * @param configStr
	 *            (eg. "setting=2400-7N1")
	 * @throws IOException
	 */
	public void configureInterface(String iface, String configStr) throws IOException {
		if (interfaces.get(iface) == null) {
			interfaces.put(iface, new MBusConnection(iface, configStr));
		}
	}

	public List<Measurement> readDeviceSync(DeviceURI url, List<ChannelIdentifier> registers) {
		return null;
	}

	public void writeChannel(DeviceURI url, ChannelIdentifier register, double value)
			throws MethodNotImplementedException {
		throw new MethodNotImplementedException();
	}

	
	public void shutdown() {
		for (MBusConnection connection : interfaces.values()) {
			connection.close();
		}
	}
	
	private void logStackTrace(Exception e) {
		StringWriter sw = new StringWriter();
		e.printStackTrace(new PrintWriter(sw));
		logger.debug(e.getMessage() + ":\n" + sw.toString());
	}

	public List<DeviceURI> getDeviceList(BusURI url) throws MethodNotImplementedException {
		return null;
	}

	public List<ChannelInformation> getDeviceChannels(DeviceURI url)
			throws MethodNotImplementedException, IOException {
		throw new MethodNotImplementedException();
	}
} /* class MBusDriverImpl */
