package org.openmuc.modbus;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.net.*;

import org.openmuc.core.driver.MethodNotImplementedException;

import net.wimpi.modbus.*;
import net.wimpi.modbus.msg.*;
import net.wimpi.modbus.io.*;
import net.wimpi.modbus.net.*;
import net.wimpi.modbus.procimg.*;

public class ModbusConnection {
	private static final int NOT_CONNECTED = 0;
	private static final int CONNECTED = 1;
	
	private TCPMasterConnection con = null;
	private int connectionState = NOT_CONNECTED;
	private InetAddress addr;
	private int port;
	
	public ModbusConnection(String server, int server_port) 
		throws UnknownHostException, Exception
		{
			try {
			addr = InetAddress.getByName(server);
		}
		catch (UnknownHostException e) {
			throw e;
		}
		this.port = server_port;
		
		con = new TCPMasterConnection(addr);
		con.setPort(port);
		try {
			con.connect();
			this.connectionState = CONNECTED;
		}
		catch (Exception e) {
			this.connectionState = NOT_CONNECTED;
			throw e;
		}
	} /* JanitzaConnection() */
	
	public ModbusConnection(String serverIP)
		throws UnknownHostException, Exception
	{
			this(serverIP, Modbus.DEFAULT_PORT);
	}
	
	void close() {
		con.close();
	}

	public InputRegister[] readInputRegisters(int register, int count)
		throws ModbusException
		{
			InputRegister[] regs = null;
			ModbusTCPTransaction trans = new ModbusTCPTransaction(con);
			ReadInputRegistersRequest req = new ReadInputRegistersRequest(register, count);
			
			req.setTransactionID(0); //usually 0 when making a request
			/*Every device connected to the TCP/IP network  must have a unique IP address. 
			  When multiple Modbus devices share a single IP, then Modbus/TCP includes an additional
			  address called the Unit ID.*/
			req.setUnitID(1); //must be 1, otherwise janitza doesn't respond
			trans.setRequest(req);
			
			try {
				trans.execute();
				ReadInputRegistersResponse res = (ReadInputRegistersResponse) trans.getResponse();
				regs = res.getRegisters();
			}
			catch (ModbusException e) {
				throw e;
			}
			
			return regs;
		} /* readInputRegisters() */
	
	/* Almost the same Function as readInputRegisters
	 * Some Devices may not support readInputRegisters, so use this function instead.
	 */
	InputRegister[] readMultipleRegisters(int register, int count)
		throws ModbusException
		{
			InputRegister[] regs = null;
			ModbusTCPTransaction trans = new ModbusTCPTransaction(con);
			ReadMultipleRegistersRequest req = new ReadMultipleRegistersRequest(register, count);

			req.setTransactionID(0); //usually 0 when making a request
			/*Every device connected to the TCP/IP network  must have a unique IP address. 
			  When multiple Modbus devices share a single IP, then Modbus/TCP includes an additional
			  address called the Unit ID.*/
			req.setUnitID(1); //must be 1, otherwise janitza doesn't respond
			trans.setRequest(req);

			
			try {
				trans.execute();
				ReadMultipleRegistersResponse res = (ReadMultipleRegistersResponse) trans.getResponse();
				regs = res.getRegisters();
			}
			catch (ModbusException e) {
				throw e;
			}
			
			return regs;
		} /* readMultipleRegisters() */
	
	void writeOutputRegister(int ref, int value)
		throws MethodNotImplementedException
	{
		ModbusTCPTransaction trans = new ModbusTCPTransaction(con);
		WriteSingleRegisterRequest req = new WriteSingleRegisterRequest(ref, new SimpleRegister(value));		
		trans.setRequest(req);
		
		WriteSingleRegisterResponse res = (WriteSingleRegisterResponse) trans.getResponse();
	}
}
