package org.openmuc.sunnywebbox;

/*
 * Copyright Fraunhofer ISE, 2010
 * Author(s): Simon Fey
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.util.List;
import java.util.Vector;

import org.openmuc.core.driver.DataReceivedListener;
import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.Measurement;

public class ReadoutTask implements Runnable {
	
	private SunnyWebboxDevice device;

	private List<ChannelIdentifier> channels;
	
	private DataReceivedListener callback;
	
	private Object handle;

	public ReadoutTask(SunnyWebboxDevice device, List<ChannelIdentifier> channels,
			DataReceivedListener callback, Object handle) 
	{
		super();
		this.channels = channels;
		this.callback = callback;
		this.handle = handle;
		this.device = device;
	}

	public void run() {
		try {
			Vector<Measurement> meas = device.getMeasurements(channels);

			if(meas.isEmpty()) {
				callback.readoutError(handle, DataReceivedListener.NO_DATA_AVAILABLE);
			} else {
				callback.dataReceived(handle, meas);
			}
		} catch (MalformedObjectLocatorException e) {
			callback.readoutError(handle, DataReceivedListener.NO_DATA_AVAILABLE);
		} catch (NullPointerException e) {
			callback.readoutError(handle, DataReceivedListener.TIMEOUT);
		} catch (Exception e) {
			callback.readoutError(handle, DataReceivedListener.TIMEOUT);
		}
	} /* run() */
}
