package org.openmuc.wago;

import net.wimpi.modbus.procimg.*;

public class Wago750DeviceNode {
	
	public int type;
	public String description;
	public int startAddress;
	public int dataLength; 
	public boolean output = false;
	public boolean digital = false;
	public int channels; /* number of channels for digital io */
	public int digitalAddress; /* coil/digital in address for digital outputs and inputs */
	
	protected WagoConnection con;
	
	public int[] getValues() {
		InputRegister regs[] = null;
		int[] ret = null;
		
		try {
			if (!digital) {
				switch (type) {
				case 638:
					regs = con.readInputRegisters(startAddress, dataLength);
					ret = new int[2];
					ret[0] = regs[1].getValue();
					ret[1] = regs[3].getValue();
					break;
				case 453:
				case 455:
				case 457:
				case 459:
				case 460:
				case 468:
					regs = con.readInputRegisters(startAddress, dataLength);
					ret = new int[4];
					ret[0] = regs[0].getValue();
					ret[1] = regs[1].getValue();
					ret[2] = regs[2].getValue();
					ret[3] = regs[3].getValue();
					break;
				case 452:
				case 454:
				case 478:
				case 476:
				case 465:
				case 466:
				case 467:
				case 461:
				case 470:
				case 472:
				case 473:
				case 474:
				case 477:
				case 479:
				case 480:
					regs = con.readInputRegisters(startAddress, dataLength);
					ret = new int[2];
					ret[0] = regs[0].getValue();
					ret[1] = regs[1].getValue();
					break;
				default:
				}
			}
			else { /* digital I/O */
				regs = con.readInputRegisters(startAddress, 1);
				ret = new int[1];
				ret[0] = regs[0].getValue();
			}
		}
		catch (Exception e) {
			
		}
		
		return ret;
	} /* getValues() */

	public void writeReg(int bitNr, double value) {
		if (!digital) {
				con.writeOutputRegister(startAddress + bitNr, (int) value);
		}
		else {
			if (value > 0.5)
				con.writeCoil(digitalAddress + bitNr, true);
			else 
				con.writeCoil(digitalAddress + bitNr, false);
		}
	}

	public int getType() {
		return type;
	}

	public String getDescription() {
		return description;
	}

	public int getStartAddress() {
		return startAddress;
	}

	public int getDataLength() {
		return dataLength;
	}

	public boolean isOutput() {
		return output;
	}

	public boolean isDigital() {
		return digital;
	}

	public int getChannels() {
		return channels;
	}

	public int getDigitalAddress() {
		return digitalAddress;
	}
	
} /* class Wago750DeviceNode */
