/**
 * WagoConnection.java
 * 
 * Modbus/TCP demonstration program
 * 
 */

package org.openmuc.wago;

import java.net.*;

import net.wimpi.modbus.*;
import net.wimpi.modbus.msg.*;
import net.wimpi.modbus.io.*;
import net.wimpi.modbus.net.*;
import net.wimpi.modbus.util.*;
import net.wimpi.modbus.procimg.*;


public class WagoConnection {
	private static final int NOT_CONNECTED = 0;
	private static final int CONNECTED = 1;
	
	private TCPMasterConnection con = null;
	private int connectionState = NOT_CONNECTED;
	private InetAddress addr;
	private int port;
	
	WagoConnection(String server, int server_port) 
		throws UnknownHostException, Exception
	{
		try {
			addr = InetAddress.getByName(server);
		}
		catch (UnknownHostException e) {
			throw e;
		}
		this.port = server_port;
		
		con = new TCPMasterConnection(addr);
		con.setPort(port);
		try {
			con.connect();
			this.connectionState = CONNECTED;
		}
		catch (Exception e) {
			this.connectionState = NOT_CONNECTED;
			throw e;
		}
	} /* WagoConnection() */
	
	WagoConnection(String server) 
		throws UnknownHostException, Exception
	{
		this(server, Modbus.DEFAULT_PORT);
	} /* WagoConnection() */
	
	void close() {
		con.close();
	}
	
	InputRegister[] readInputRegisters(int register, int count)
		throws ModbusException
	{
		InputRegister[] regs = null;
		ModbusTCPTransaction trans = new ModbusTCPTransaction(con);
		ReadInputRegistersRequest req = new ReadInputRegistersRequest(register, count);
		
		trans.setRequest(req);
		
		try {
			trans.execute();
			ReadInputRegistersResponse res = (ReadInputRegistersResponse) trans.getResponse();
			regs = res.getRegisters();
		}
		catch (ModbusException e) {
			throw e;
		}
		
		return regs;
	} /* readInputRegisters() */
	
	InputRegister[] getInputRegisters(int ref, int count) throws Exception {
		ModbusTCPTransaction trans = new ModbusTCPTransaction(con);
		ReadInputRegistersRequest req = new ReadInputRegistersRequest(ref, count);
		
		trans.setRequest(req);

		trans.execute();
		
		ReadInputRegistersResponse res = (ReadInputRegistersResponse) trans.getResponse();
		return res.getRegisters();
	}
	
	void writeOutputRegister(int ref, int value) {
		ModbusTCPTransaction trans = new ModbusTCPTransaction(con);
		WriteSingleRegisterRequest req = new WriteSingleRegisterRequest(ref, new SimpleRegister(value));		
		trans.setRequest(req);
		try {
			trans.execute();
			WriteSingleRegisterResponse res = (WriteSingleRegisterResponse) trans.getResponse();
		} catch (ModbusIOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ModbusSlaveException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ModbusException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public boolean[] readCoils(int ref, int count) throws ModbusIOException, ModbusSlaveException, ModbusException {
		ModbusTCPTransaction trans = new ModbusTCPTransaction(con);
		ReadCoilsRequest req = new ReadCoilsRequest(ref, 1);
			
		trans.setRequest(req);
		trans.execute();
		ReadCoilsResponse res = (ReadCoilsResponse) trans.getResponse();
		
		boolean[] ret = new boolean[count];
		
		BitVector bv = res.getCoils();
		
		for (int i = 0; i < count; i++) {
			ret[i] = bv.getBit(i);
		}
		
		return ret;	
	}
	
	public void writeCoil(int ref, boolean value) {
		ModbusTCPTransaction trans = new ModbusTCPTransaction(con);
		WriteCoilRequest req = new WriteCoilRequest(ref, value);
		
		trans.setRequest(req);
		try {
			trans.execute();
			ModbusResponse res = trans.getResponse();
		} catch (ModbusIOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ModbusSlaveException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ModbusException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

} /* class WagoConnection */
