package org.openmuc.wago;

import java.io.IOException;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

import org.apache.log4j.Logger;
import org.openmuc.core.driver.BusURI;
import org.openmuc.core.driver.ChannelInformation;
import org.openmuc.core.driver.DataReceivedListener;
import org.openmuc.core.driver.DeviceDriverInterface;
import org.openmuc.core.driver.DeviceURI;
import org.openmuc.core.driver.MethodNotImplementedException;
import org.openmuc.core.driver.NoSuchInterfaceException;
import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.Measurement;

public class WagoDriverImpl implements DeviceDriverInterface {
	private static Logger logger = Logger.getLogger(WagoDriverImpl.class);
	
	private static int instanceCount = 0;
	
	private final String[] supportedOperations = {"readMeterAsync","writeRegister"};
	
	private int instanceID = 0;
	private int threadCount = 0;
	
	private Executor executor;
	
	private Vector<Wago750Device> devicePool = new Vector<Wago750Device>();
	
	public WagoDriverImpl() {
		instanceID = instanceCount++;
		
		/* Create new thread pool */
		executor = Executors.newCachedThreadPool(new ThreadFactory() {
			public Thread newThread(Runnable arg0) {
				return new Thread(arg0, "wagothreadpool-" + instanceID + "-" + threadCount++);
			}
		});
	}

	public void configureInterface(String iface, String configStr)
			throws NoSuchInterfaceException, MethodNotImplementedException
	{
		throw new MethodNotImplementedException();
	}

	
	public String getDriverID() {
		return "wagoio";
	}

	
	public List<DeviceURI> getDeviceList(BusURI url)
			throws MethodNotImplementedException 
	{
		throw new MethodNotImplementedException();
	}

	
	public Vector<String> getSupportedOperations() {
		Vector<String> operations = new Vector<String>();
		
		for (String operation : supportedOperations) {
			operations.add(operation);
		}
		
		return operations;
	}

	
	public void readDeviceAsync(DeviceURI url,
			List<ChannelIdentifier> registers,
			DataReceivedListener callback, Object handle)
			throws MethodNotImplementedException 
	{
		Wago750Device device = null;
		
		try {
			device = getDeviceFromPool(url);
		} catch (IOException e) {
			logger.error(e.getMessage());
		}
						
		executor.execute(new ReadoutThread(device, registers, callback, handle));
	}

	
	public List<Measurement> readDeviceSync(DeviceURI url,
			List<ChannelIdentifier> registers)
			throws IOException 
	{	
		Wago750Device device = null;
		
		try {
			device = getDeviceFromPool(url);
		} catch (IOException e) {
			logger.error(e.getMessage());
		}
		
		Vector<Measurement> meas;
		try {
			meas = device.getChannels(registers);
		} catch (Exception e) {
			throw new IOException("Cannot connect to " + url.toString());
		}
		return meas;
	}

	
	public void writeChannel(DeviceURI url, ChannelIdentifier register,
			double value) throws MethodNotImplementedException, IOException 
	{
		Wago750Device device = null;
				
		device = getDeviceFromPool(url);
		
		device.writeOutputChannel(register, value);
	} /* writeRegister() */

	
	private Wago750Device getDeviceFromPool(DeviceURI url) throws IOException {
		Wago750Device device = null;
		
		for (Wago750Device availableDevice : devicePool) {		
			if (availableDevice.getUrl().getAddress().equals(url.getAddress())) {
				device = availableDevice;
				break;
			}
		}
		
		if (device == null) {
			try {
				device = new Wago750Device(url);
				devicePool.add(device);
			} catch (Exception e) {
				throw new IOException("Cannot connect to " + url.toString());		
			}
		}
		
		return device;
	} /* getDeviceFromPool() */

	public List<ChannelInformation> getDeviceChannels(DeviceURI url)
			throws MethodNotImplementedException, IOException {
		throw new MethodNotImplementedException();
	}

}
