package org.openmuc.wago;


/**
 * 
 * Object locators for WAGO I/O system are composed of
 *  * module number
 *  * weather it is an input or output
 *  * and the channel
 *  
 *  e.g. first module (slot), first input channel
 *  "n0.input(0)"
 *  
 *  e.g. second module (slot), second output channel
 *  "n1.output(1)"
 * 
 * @author mzillgit
 *
 */
public class WagoObjectLocator {
	
	private int moduleNo;
	private boolean input;
	private int channel;
	
	public WagoObjectLocator(String locator) throws MalformedObjectLocatorException {
		
		if (locator.startsWith("n")) {
			int offsetPos = locator.indexOf('.');
			try {
				this.moduleNo = Integer.parseInt(locator.substring(1, offsetPos));
			} catch (NumberFormatException e) {
				throw new MalformedObjectLocatorException("");
			}
			
			int ioPos = locator.indexOf('(');
			String ioStr = locator.substring(offsetPos + 1, ioPos);
			
			if (ioStr.equals("input")) this.input = true;
			else if (ioStr.equals("output")) this.input = false;
			else throw new MalformedObjectLocatorException("");
			
			int channelStart = locator.indexOf('(');
			int channelStop = locator.indexOf(')');
			
			try {
				this.channel = Integer.parseInt(locator.substring(channelStart + 1, channelStop));
			} catch (NumberFormatException e) {
				throw new MalformedObjectLocatorException("");
			}
		}
		else throw new MalformedObjectLocatorException("");
	}
		
	public int getModuleNo() {
		return moduleNo;
	}

	public boolean isInput() {
		return input;
	}

	public int getChannel() {
		return channel;
	}
}
