package org.openmuc.ehzwebtool.internal;

/*
 * Copyright Fraunhofer ISE, 2010 - 2011
 * Author(s): Michael Zillgith
 *            Frederic Robra
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * openMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * openMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.io.IOException;
import java.net.URL;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmuc.core.common.SerialPort;
import org.openmuc.core.measurement.Unit;
import org.openmuc.core.datamanager.config.ChannelConfiguration;
import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.datamanager.config.DeviceConfiguration;
import org.openmuc.ehz.driver.Device;
import org.openmuc.ehz.driver.DeviceInfo;
import org.openmuc.ehz.iec62056_21.IECDevice;
import org.openmuc.ehz.sml.SMLDevice;
import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;
import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.BundleContext;

public class EHZWebToolPlugin implements WebInterfacePlugin {

	private BundleContext context;
	private ResourceLoader loader;
	private ConfigurationService configService;
	private List<DeviceInfo> deviceInfos = null;
	private String type;
	private String iface;
	
	private class Register {
		public Register(String label, String description, Unit unit) {
			this.label = label;
			this.description = description;
			this.unit = unit;
		}
		
		public String label;
		public String description;
		public Unit unit;
	}
	
	public EHZWebToolPlugin(BundleContext context, ConfigurationService configService) {
		this.context = context;
		this.configService = configService;
		loader = new ResourceLoader(context);
	}
	
	public Hashtable<String,String> getResources() {
		Hashtable<String,String> resources = new Hashtable<String,String>();
		
		resources.put("css", "css");
		
		return resources;
	}
	
	public String getApplicationAlias() {
		return "ehzwebtool";
	}

	public String getApplicationName() {
		return "eHZ Tool";
	}

	public String getApplicationDescription() {
		return "Tool to read eHZ.";
	}
	
	private void configureDevice(HashMap<String, Register> registers, HttpServletRequest request) {
		String vendor = request.getParameter("meterName");
		String id = request.getParameter("meterID");
		String deviceName = vendor.replace(" ", "-") + "." + id;
		DeviceConfiguration newMeter = configService.createDevice(deviceName);
		newMeter.setDescription("autogenerated");
		
		newMeter.setMeterUrl("ehz:" + iface + ":?type=" + type);
		newMeter.setVendorID(vendor);
		newMeter.setDeviceID(id);
		newMeter.setSamplingPeriod(5000);

		Calendar cal = new GregorianCalendar();
		cal.set(Calendar.YEAR, 2010);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.DAY_OF_MONTH, 1);
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		
		for (String obisCode : registers.keySet()) {
			Register register = registers.get(obisCode);
			
			ChannelConfiguration newRegister;
			
			if ((register.label != null) && (!register.label.equals(""))) {
				newRegister = newMeter.createChannel(register.label);
			}
			else {
				newRegister = newMeter.createChannel(deviceName + "-" + obisCode);
			}
			
			newRegister.setObjectLocator(obisCode);
			newRegister.setStoringPeriod(60);
			newRegister.setStorageType(ChannelConfiguration.STORE_TYPE_MEAN);
			newRegister.setDescription(register.description);
			newRegister.setUnit(register.unit);
			
			newRegister.setStoringStartTime(cal.getTime());
		}
		
		configService.persist();
	}


	public View getContentView(HttpServletRequest request, PluginContext context) {
		String template = "";
		
		if (context.getLocalPath().equals("/readmeter")) {
			iface = request.getParameter("serialdevice");
			String config = request.getParameter("serialconfig");
			
			String message = "";
			type = "";
			Device device = null;
			try {
				device = new IECDevice(iface, config);
				deviceInfos = device.getDeviceInfos();
				type = "IEC";
			}
			catch (Exception e) {
				device.close();
				device = null;
				try {
					device = new SMLDevice(iface, config);
					deviceInfos = device.getDeviceInfos();
					type = "SML";
				}
				catch (Exception er) { }
			}
			finally {
				device.close();
			}
			
			EHZView view;
			if(!type.equals("")) {
				template = loader.getResourceAsString("modulesview.html");
				view = new EHZView(template);
				view.addToContext("meter", device);
				view.addToContext("infos", deviceInfos);
				view.addToContext("units", Unit.values());
			}
			else {
				template = loader.getResourceAsString("message.html");
				view = new EHZView(template);
				message = "Meter not found!";
			}

			view.addToContext("Message", message);
			return view;
		}
		else if (context.getLocalPath().equals("/addmeter")) {
			template = loader.getResourceAsString("message.html");
			EHZView view = new EHZView(template);
			
			HashMap<String, Register> registers = new HashMap<String, Register>();
			
			String[] params = request.getParameterValues("registers");
			String message = "Configured!";
			if (params != null) {
				for (String param : params) {
					Register register = new Register(
							request.getParameter(param + "label"), 
							request.getParameter(param + "description"),
							Unit.fromUnitCode(new Integer(request.getParameter(param + "unitCode"))));
					
					registers.put(param, register);
				}
			}
			else {
				message = "No channels selected!";
			}

			try {
				configureDevice(registers, request);
			} catch (Exception e) {
				message = "Configuration failed!";
			}
			
			view.addToContext("Message", message);
			return view;
		}
		else {
			template = loader.getResourceAsString("enterMeterInfo.html");
			List<String> portList = SerialPort.getPortIdList();
			
			EHZView view = new EHZView(template);
			view.addToContext("portList", portList);
			
			return view;
		}
	}

	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		return true;
	}

	public URL getResource(String name) {	
		return context.getBundle().getResource(name);
	}

	public String getMimeType(String name) {
		return null;
	}

	public PluginCategory getCategory() {
		return PluginCategory.TOOL;
	}

}
