package org.openmuc.sunnywebtool.internal;

import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.datamanager.config.DeviceConfiguration;
import org.openmuc.core.datamanager.config.ChannelConfiguration;
import org.openmuc.core.driver.BusURI;
import org.osgi.framework.BundleContext;

public class DeviceConfigurator {
	private ConfigurationService configService;
	private SunnyWebbox sunnyWebbox;
	private BundleContext context;

	public DeviceConfigurator(ConfigurationService configService, BundleContext context) {
		this.configService = configService;
		this.context = context;
	}
	
	public void configureDevice(String ipAddress, String deviceAddress, String[] channels, String storageIntervall) {
		try {
			sunnyWebbox = new SunnyWebbox(new BusURI("sunnywebbox"+":"+ipAddress), context);
		} catch (IOException e) {
			System.err.println("Unable to configure Device. I/O Error!");
			e.printStackTrace();
		}
		
		int storageIntervallAsInt = getAsInt(storageIntervall);
		
		DeviceConfiguration newMeter = configService.createDevice(deviceAddress);
		newMeter.setDescription("autogenerated");
		newMeter.setVendorID("SunnyWebbox");
		newMeter.setDeviceID(deviceAddress);
		newMeter.setMeterUrl("sunnywebbox:"+ipAddress+":"+deviceAddress);

		for (SunnyAttachedDevice device : sunnyWebbox.getDevices()) {
			if (device.getAddress().equals(deviceAddress)) {
				for (String channel : channels) {
					ChannelConfiguration newRegister = newMeter.createChannel(deviceAddress+":"+channel);
					newRegister.setObjectLocator(channel);
					newRegister.setStoringPeriod(storageIntervallAsInt); /*seconds!*/
					newRegister.setStorageType(ChannelConfiguration.STORE_TYPE_MEAN);
					
					Calendar cal = new GregorianCalendar();
					cal.set(Calendar.YEAR, 2010);
					cal.set(Calendar.MONTH, 0);
					cal.set(Calendar.DAY_OF_MONTH, 1);
					cal.set(Calendar.HOUR_OF_DAY, 0);
					cal.set(Calendar.MINUTE, 0);
					cal.set(Calendar.SECOND, 0);
					cal.set(Calendar.MILLISECOND, 0);
					newRegister.setStoringStartTime(cal.getTime());
				}
			}
		}

		configService.persist();
	}
	
	private int getAsInt(String intervall) {
		if(intervall.matches("^[0-9]+$")) {
			try {
				return 60*Integer.parseInt(intervall); /*seconds!*/
			} catch (NumberFormatException e) {
				return 300; /*5 min Intervall if no valid literal is given*/
			}
		}
		return 300; /*5 min Intervall if no valid literal is given*/
	}
}
