package org.openmuc.sunnywebtool.internal;

import java.io.IOException;
import java.net.SocketTimeoutException;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.openmuc.core.driver.BusURI;
import org.openmuc.core.driver.DeviceDriverManager;
import org.openmuc.core.driver.DeviceURI;
import org.openmuc.core.driver.MethodNotImplementedException;
import org.openmuc.core.driver.NoSuchDriverException;
import org.openmuc.core.measurement.ChannelIdentifier;
import org.openmuc.core.measurement.Measurement;
import org.openmuc.core.measurement.Value;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

public class SunnyWebbox {
	
	private ServiceReference serviceRef;
	private DeviceDriverManager deviceDriverManager;
	private List<SunnyAttachedDevice> devices;
	private BusURI url;
	
	public SunnyWebbox(BusURI url, BundleContext context) throws IOException {
		serviceRef = context.getServiceReference(DeviceDriverManager.class.getName());
		deviceDriverManager = (DeviceDriverManager) context.getService(serviceRef);
		devices = new Vector<SunnyAttachedDevice>();
		List<ChannelIdentifier> chanID = new Vector<ChannelIdentifier>();
		
		try {
			for (DeviceURI sunnyattached : deviceDriverManager.getConnectedDevices(url)) {
				devices.add(new SunnyAttachedDevice(sunnyattached, deviceDriverManager.getDeviceChannels(sunnyattached)));
			}
		} catch (MethodNotImplementedException e) {
			// will never be thrown
			e.printStackTrace();
		} catch (NoSuchDriverException e) {
			System.err.println("Sunny Webbox Driver not present");
			e.printStackTrace();
		}
		
		if(!devices.isEmpty()) {
			for(SunnyAttachedDevice dev : devices) {
				List<Measurement> measurements = null;
				for(ExtendedChannelInformation extChanInf : dev.getChannels()) {
					chanID.add(new ChannelIdentifier(dev.getAddress()+":"+extChanInf.getName(), extChanInf.getName()));
				}
				DeviceURI devURL = new DeviceURI(url.toString()+":"+dev.getAddress());
				try {
					measurements = deviceDriverManager.readDeviceSync(devURL, chanID);
				} catch (MethodNotImplementedException e) {
					//is implemented!
					e.printStackTrace();
				}
				if (measurements!=null) {
				HashMap<String, Measurement> listJoin = new HashMap<String, Measurement>();
					for (Measurement meas : measurements) {
						listJoin.put(meas.getDataID().getObjectLocator(), meas);
					}
					for (ExtendedChannelInformation extChanInf : dev.getChannels()) {
						Measurement meas = listJoin.get(extChanInf.getName());
						if(meas != null) {
							Value meteredVal = meas.getValue();
							if(meteredVal != null) {
								double doubleVal = meteredVal.getValue();
								extChanInf.setCurValue(doubleVal);
							}
						}
					}
				}
			}
		}
	}
	
	public List<SunnyAttachedDevice> getDevices() {
		return this.devices;
	}
}
