package org.openmuc.wagowebtool.internal;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmuc.core.datamanager.config.ConfigurationService;
import org.openmuc.core.measurement.Unit;
import org.openmuc.wago.Wago750Device;
import org.openmuc.wago.Wago750DeviceNode;
import org.openmuc.webinterface.PluginCategory;
import org.openmuc.webinterface.PluginContext;
import org.openmuc.webinterface.ResourceLoader;
import org.openmuc.webinterface.View;
import org.openmuc.webinterface.WebInterfacePlugin;
import org.osgi.framework.BundleContext;

public class WagoWebToolPlugin implements WebInterfacePlugin {

	private BundleContext context;
	private ResourceLoader loader;
	private DeviceConfigurator deviceConfigurator;
	private Wago750DeviceNode[] nodes;
	private ArrayList<RegisterInfo> registers;
	
	public WagoWebToolPlugin(BundleContext context, ConfigurationService configService) {
		this.context = context;
		loader = new ResourceLoader(context);
		deviceConfigurator = new DeviceConfigurator(configService);
	}
	
	public Hashtable<String,String> getResources() {
		Hashtable<String,String> resources = new Hashtable<String,String>();
		
		resources.put("css", "css");
		
		return resources;
	}
	
	public String getApplicationAlias() {
		return "wagowebtool";
	}

	public String getApplicationName() {
		return "Wago I/O Tool";
	}
	
	public String getApplicationDescription(){
		return "Tool to connect to a Wago";
	}

	public ArrayList<RegisterInfo> readWago(String host) throws Exception{
		Wago750Device device;
		
		System.out.println("Connecting to wago io system ...");
		try {
			device = new Wago750Device((String)host);
			
			nodes = device.getNodes();

			for (int i = 0; i < nodes.length; i++) {
				System.out.printf("Module %d\t%04x\t%s\n", nodes[i].type,
						nodes[i].startAddress, nodes[i].description);

				int values[] = nodes[i].getValues();
				if (values != null) {
					for (int k = 0; k < values.length; k++) {
						System.out.printf("\tChannel %d: %d\n", k, values[k]);
					}
				}

			}
			
			device.disconnect();
			
		} catch (Exception e) {
			System.out.println("Unable to connect to host " + host);
			throw new Exception("Unable to connect to host " + host);
		}
		ArrayList<RegisterInfo> registers=new ArrayList<RegisterInfo>();
		
		for (int nodeCount = 0; nodeCount < nodes.length; nodeCount++) {
			Wago750DeviceNode node = nodes[nodeCount];
			int channels = node.getChannels();
			for (int i = 0; i < channels; i++) {
				String channelName;
				String analogDigital;
				String objectLocator;
				
				if (node.isDigital())
					analogDigital = "d";
				else 
					analogDigital = "a";
				
				if (node.isOutput()) {
					channelName = "WagoIO-" + host + "-n" + nodeCount + "." + analogDigital + "o" + i;
					objectLocator = "n" + nodeCount + ".output(" + i + ")";
				}
				else {
					channelName = "WagoIO-" + host + "-n" + nodeCount + "." + analogDigital + "i" + i;
					objectLocator = "n" + nodeCount + ".input(" + i + ")";
				}
				registers.add(new RegisterInfo(channelName,node,0,objectLocator));
			}
		}
		return registers;
	}
	
	public View getContentView(HttpServletRequest request, PluginContext context) {
		String template = "";
		
		if (context.getLocalPath().equals("/configure")) {
			String ipAddress = request.getParameter("ip");
			template = loader.getResourceAsString("message.html");
			WagoView view = new WagoView(template);
			
			
			try {
				ArrayList<RegisterInfo> registers=new ArrayList<RegisterInfo>();
			
				String[] params = request.getParameterValues("registers");
				
				for (String param : params) {
	
					RegisterInfo register = new RegisterInfo(
							request.getParameter(param + "label"), 
							request.getParameter(param + "description"),
							new Integer(request.getParameter(param + "unitCode")),
							param);
					registers.add(register);
				}
				deviceConfigurator.configureDevice(registers, ipAddress, nodes);
				view.addToContext("wagoMessage", "Configured device with IP " + ipAddress);
			}
			catch (Exception e) {
				view.addToContext("wagoMessage", "Unable to configure device");
			}

			return view;
		}
		else if (context.getLocalPath().equals("/listmodules")) {
			WagoView view = null;
			
			try {
				template = loader.getResourceAsString("modulesview.html");
				view = new WagoView(template);
				String ip = request.getParameter("ipaddress");
				registers = readWago(ip);
				
				view.addToContext("wagoIP", ip);
				view.addToContext("registers", registers);
				view.addToContext("nodes", nodes);
				view.addToContext("units", Unit.values());
			}
			catch (Exception e) {
				template = loader.getResourceAsString("message.html");
				view = new WagoView(template);
				view.addToContext("wagoMessage", e.getMessage());
			}
			
			return view;

		
		}
		else {
			template = loader.getResourceAsString("enterIP.html");
			return new WagoView(template);
		}
	}

	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		return true;
	}

	public URL getResource(String name) {	
		return context.getBundle().getResource(name);
	}

	public String getMimeType(String name) {
		return null;
	}

	public PluginCategory getCategory() {
		return PluginCategory.TOOL;
	}

}
