package org.openmuc.webconfigadmin.internal;

import java.io.IOException;
import java.util.Dictionary;
import java.util.Enumeration;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
import org.osgi.service.cm.Configuration;
import org.osgi.service.cm.ConfigurationAdmin;
import org.osgi.util.tracker.ServiceTracker;

public class ConfigAdminTracker extends ServiceTracker {
	
	private ConfigurationAdmin confAdmin = null;

	public ConfigAdminTracker(BundleContext context) {
		super(context, ConfigurationAdmin.class.getName(), null);
	}
	
	public ConfigurationAdmin getConfAdmin() {
		return confAdmin;
	}

	@Override
	public Object addingService(ServiceReference reference) {
		ConfigurationAdmin configAdminService = (ConfigurationAdmin) context.getService(reference);
		
		System.out.println("Found ConfigurationAdmin: " + configAdminService);
		confAdmin = configAdminService;
		
		try {
			Configuration[] configs = confAdmin.listConfigurations(null);
			
			if (configs != null) {
				for (Configuration config : configs) {
					System.out.println("----- Configuration -----");
					System.out.println("PID: " + config.getPid());
					System.out.println("Bundle location: " + config.getBundleLocation());
					System.out.println("Properties: ");
					Dictionary<?, ?> properties = config.getProperties();
					Enumeration<?> keys = properties.keys();
					while (keys.hasMoreElements()) {
						String key = (String) keys.nextElement();
						String value = (String) properties.get(key);
						System.out.println(key + ": " + value);
					}
				}
			}
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidSyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return configAdminService;
	}

	@Override
	public void removedService(ServiceReference reference, Object service) {
		ConfigurationAdmin configAdminService = (ConfigurationAdmin) service;
		
		System.out.println("Remove ConfigurationAdmin: " + configAdminService);
		confAdmin = null;
		
		context.ungetService(reference);
	}
}
