package org.openmuc.webinterface;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import org.osgi.framework.BundleContext;


/**
 * Helper class to load resources from the bundle classpath
 * 
 * @author mzillgit
 *
 */
public class ResourceLoader {
	
	private BundleContext context;
	
	/**
	 * 
	 * @param context
	 */
	public ResourceLoader(BundleContext context) {
		this.context = context;
	}
	
	/**
	 * 
	 * Get resource as URL
	 * 
	 * @param name Resource name relative to bundles classpath
	 * @return URL object of the requested resource.
	 */
	public URL getResource(String name) {
		return context.getBundle().getResource(name);
	}
	
	/**
	 * 
	 * Get a text file resource as String
	 * 
	 * @param name Resource name relative to bundles classpath
	 * @return String representation of the requested resource. null if resource is not available.
	 */
	public String getResourceAsString(String name) {
		URL url = getResource(name);
		
		try {
			InputStream stream = (InputStream) url.getContent();
			
			BufferedReader reader = new BufferedReader(new InputStreamReader(stream));
			
			String line = null;
			StringBuilder builder = new StringBuilder();
			
			while ((line = reader.readLine()) != null) {
				builder.append(line);
				builder.append('\n');
			}
			
			reader.close();
			
			return builder.toString();
		} catch (IOException e) {
			return null;
		}
		
	}
		
}
